<?php
/**
 * Connected
 *
 * @license		GPL
 * @package		Connected
 * @link		https://gitlab.liris.cnrs.fr/mloiseau/mw_connected
 *
 **/
class ConnectedHooks {
	private static $user = false;

	//try to get the user anyway I can
	//through user or parser or output or wgUser…
	//and set it for next time
	private static function get_user($parserOrOther=null){
		if(self::$user === false){
			$res = false;
			if (!is_null($parserOrOther)){
				if(get_class($parserOrOther) == "User"){
					$res = $parserOrOther;
				}
				else if(get_class($parserOrOther) == "Parser"){
					//cf. https://github.com/wikimedia/mediawiki-extensions-WikiTextLoggedInOut/blob/d764bd3972ed6b84a4af3ed423a3a5fd9b82c493/includes/WikiTextLoggedInOut.php#L39
					if ( method_exists( $parserOrOther, 'getUserIdentity' ) ) {
						// MW 1.36+
						$res = $parserOrOther->getUserIdentity();
					}
					else {
						if (method_exists( $parserOrOther, 'getUser' )){
							$res = $parserOrOther->getUser();
						}
					}
				}
				else if(get_class($parserOrOther) == "OutputPage"){
					$res = $parserOrOther->getUser();
				}
			}
			if($res === false){
				global $wgUser;
				$res = $wgUser;
			}
			self::$user = $res;
		}
		else{
			$res = self::$user;
		}
		return $res;
	}

	// Register any render callbacks with the parser
	public static function onParserFirstCallInit( Parser $parser ) {
		// Create a function hook associating the "example" magic word with renderExample()
		$parser->setFunctionHook( 'iflogged', [ self::class, 'ifConnectedRender' ] );
	}

	// Render the output of {{#ifloggedin:}} i.e. iflogged function.
	public static function ifConnectedRender( Parser $parser, $ifLoggedIn = '', $ifNot = '') {
		// The input parameters are wikitext with templates expanded.
		// The output should be wikitext too.
		if (!self::get_user($parser)->isRegistered()) {
			$output = $ifNot;
		}
		else{
			$output = $ifLoggedIn;
		}
		return [ $output, 'noparse' => false ];
	}

	//calcul des variables
	public static function wfConnectedAssignValue( Parser $parser, &$cache, $magicWordId, &$ret, $frame) {
		$result = false;
		switch($magicWordId){
			case 'connected' :
				if (!self::get_user($parser)->isRegistered()) {
					$ret = 0;
				}
				else{
					$ret = 1;
				}
				$result = true;
				break;
			case 'u_name':
				if (!self::get_user($parser)->isRegistered()) {
					//$ret = self::$theUser->getName();
					$ret = self::get_user($parser)->getName();
				}
				else{
					//$ret = self::$theUser->getName();
					$ret = self::get_user($parser)->getName();
				}
				$resutl = true;
				break;
		}
		return $result;
	}

	//mettre à jour la liste de variables
	public static function connectedDeclareVarIds( &$customVariableIds ) {
		// $customVariableIds is where MediaWiki wants to store its list of custom
		// variable IDs. We oblige by adding ours:
		$customVariableIds[] = 'connected';
		$customVariableIds[] = 'u_name';
	}

	#Cacher la barre de menu quand on n'est pas logué et charger une css supplémentaire
//https://www.mediawiki.org/wiki/Manual:Interface/Sidebar#Change_sidebar_content_when_logged_in_(PHP)
//https://www.mediawiki.org/wiki/Snippets/Load_JS_and_CSS_by_URL
	public static function lfHideSidebar( $skin, &$bar ) {
		// Hide sidebar for anonymous users
		if ( !self::get_user()->isRegistered() ) {
			$bar = array(
				'navigation' => array(
					array(
						'text'   => wfMessage( 'login' ) -> text(),
						'href'   => SpecialPage::getTitleFor( 'Login' )->getLocalURL(),
						'id'     => 'n-login',
						'active' => ''
					)
				)
			);
		}
		return true;
	}

	#Ajouter une feuille de style quand pas connecté
	public static function add_NotLoggedIncss( OutputPage $out, Skin $skin ) {
		if ( !self::get_user($out)->isRegistered() ) {
			$out->addStyle('/w/index.php?title=MediaWiki:NotLoggedIn.css&action=raw&ctype=text/css');
			$out->addMeta( "logged", "no" );
		}
		else{
			$out->addStyle('/w/index.php?title=MediaWiki:LoggedIn.css&action=raw&ctype=text/css');
			$out->addMeta( "logged", "yes" );
		}
	}

	// *** use a specific cache hash key for registered users
	// so the cache of a page is always related to anonymous/registered_users
	//https://github.com/thomas-topway-it/mediawiki-extensions-PageOwnership/blob/13c379e0bac178a963d08a63fd776a30fa85a339/includes/PageOwnership.php#L326
	public static function onPageRenderingHash( &$confstr, User $user, &$forOptions ) {
		// *** for some reason we cannot rely on $user->isRegistered()
		if ( $user->isRegistered() ) {
			$confstr .= '+registered_user';
		}

	}
}
